###############################################################################
# Sample code for main thresholds analysis
# Sample data includes data for two stations in the Southwest heat alert area
###############################################################################

# Set the working directory

  setwd("../sample_code/")

# Load packages

  library(tidyverse)
  library(splines)
  library(meta)

# Load the sample data

  load("data/sample_data.Rdata")
  
# Define the heat alert thresholds for this heat alert area
  
  # A range of high-low-high threshold combinations to test different thresholds
  threshold_combos_Southwest <- expand.grid(high = c(28:30), low = c(15:17))
  # The finalized thresholds for this heat alert area
  high_threshold_Southwest <- 29
  low_threshold_Southwest <- 16
  
# Character vector of station names included in the sample analysis
  
  stations <- c("Victoria", "Vancouver")

# Source the code file containing functions

  source("code/functions.R")

# Perform the analysis for each threshold high-low-high combination and station
  
  analysis_results <- data.frame(threshold_combos_Southwest, 
                                 "station" = rep(stations, 
                                                 each = nrow(threshold_combos_Southwest)),
                                 stringsAsFactors = F) %>%
    
    # 1. Group the data by high threshold, low threshold, and station and 
    # nest the data frame
    
    group_by(high, low, station) %>%
    nest() %>%
    
    # 2. Assign the data for each station to each row
    
    mutate(data = map(station, ~ filter(sample_data, station == .x))) %>%
    
    # 3. Assign each date in the data for each station to a heat alert category
    # based on each high-low-high threshold combination
    
    mutate(category = pmap(list(x = data, y = high, z = low), function(x, y, z){
      assign_category(high_temp1 = x$forecast_high_dayt,
                      high_temp2 = x$forecast_high_daytp1,
                      low_temp = x$forecast_low_daytp1, 
                      high_threshold = y,
                      low_threshold = z)})) %>%
    # Unnest, group, and re-nest the data frame to get the column created above into
    # the data df for each row
    unnest() %>%
    group_by(high, low, station) %>%
    nest() %>%
    
    # 4. Model the data
  
    mutate(model_fit = map(data, ~ model_the_data(dataset = .x, 
                                                  response_var = "mortality_3daysum"))) %>%
      
    # 5. Create a data frame of information from the model for each station
    
    mutate(fit_results_df = pmap(list(x = model_fit, y = high, z = low), function(x, y, z){
      create_results_df(high_threshold = y,
                        low_threshold = z,
                        model_data = x)}))
  
# Do the meta-analysis for the heat alert area for each high-low-high combination
  
  meta_analysis_results <- analysis_results %>%
    group_by(high, low) %>%
    nest() %>%
    mutate(fit_results_df = map(data, ~ bind_rows(.x$fit_results_df)),
           ma_results = map(fit_results_df, ~ perform_metaanalysis(dataset = .x,
                                                                   haa = "Southwest"))) %>%
    select(high, low, ma_results) %>%
    unnest() %>%
    mutate(category = factor(category, levels = c("Category 2", "Category 1", "Category 0")))
  
# Visualize the results
  
  for(current in stations){
    
    station_data <- analysis_results %>%
      filter(station == current) %>%
      pull(fit_results_df) %>%
      bind_rows()
    
    station_plot <- ggplot() +
      geom_pointrange(data = station_data,
                      aes(x = category, y = RR, ymin = LCI, ymax = UCI), 
                      size = 0.4) +
      scale_x_discrete(breaks = c("Category 2", "Category 1", "Category 0"), labels = c(2:0)) +
      facet_grid(high_threshold ~ low_threshold, drop = F) +
      geom_hline(yintercept = 1.00, colour = "red") +
      ylab("Relative Rate") +
      xlab("Heat Alert Category") +
      ggtitle(paste0("Southwest - ", current)) +
      theme(plot.title = element_text(hjust = 0.5))
    
    ggsave(filename = paste0(current, ".png"), 
           plot = station_plot,
           path = "output/")
    
  }
  
  rm(current, station_data)
  
  Southwest_plot <- ggplot() +
    geom_pointrange(data = meta_analysis_results,
                    aes(x = category, y = estimate, ymin = lci, ymax = uci), 
                    size = 0.4) +
    scale_x_discrete(breaks = c("Category 2", "Category 1", "Category 0"), labels = c(2:0)) +
    facet_grid(high ~ low, drop = F) +
    geom_hline(yintercept = 1.00, colour = "red") +
    ylab("Relative Rate") +
    xlab("Heat Alert Category") +
    ggtitle("Southwest") +
    theme(plot.title = element_text(hjust = 0.5))
  
  ggsave(filename = "Southwest.png", 
         plot = Southwest_plot,
         path = "output/")